@@ -28,12 +28,10 @@ class Version(resource.Resource):
 
     @classmethod
     def list(cls, session, paginated=False, base_path=None, **params):
-
         if base_path is None:
             base_path = cls.base_path
 
-        resp = session.get(base_path,
-                           params=params)
+        resp = session.get(base_path, params=params)
         resp = resp.json()
         for data in resp[cls.resources_key]['values']:
             yield cls.existing(**data)
